var STATUS_OK = 0;
var STATUS_ERR = 1;

var Feed = function(data) {
	this.data = data;

	this.id = ko.observable(data.id);
	this.title = ko.observable(data.title);
	this.unread = ko.observable(parseInt(data.unread));

	this.isActive = ko.computed(function() {
		return viewModel.activeFeed() && viewModel.activeFeed().id() == this.id();
	}, this);
};

var Headline = function(data) {
	this.data = data;

	this.checked = ko.observable(false);
	this._expanded = ko.observable(false);

	this.expanded = ko.computed({
		read: function() { return viewModel.expanded() || this._expanded() },
		write: function(value) { this._expanded(value); },
		owner: this,
	});

	this.published = ko.observable(data.published);
	this.marked = ko.observable(data.marked);
	this.link = ko.observable(data.link);
	this.title = ko.observable(data.title);
	this.content = ko.observable(data.content);
	this.excerpt = ko.observable(data.excerpt);
	this.id = ko.observable(data.id);
	this.updated = ko.observable(
		dateFormat(new Date(data.updated*1000), "mmm d, yyyy h:MM"));
	this.unread = ko.observable(data.unread);

	this.catchup = function() {

		var article = this;

		api_request({"op": "updateArticle", "article_ids": this.id(),
			"mode": 0,
			"field": 2}, function() {
				article.unread(false);
				return true;
			});

		return true;
	};


	this.togglePub = function() {

		var article = this;

		api_request({"op": "updateArticle", "article_ids": this.id(),
			"mode": !article.published() ? 1 : 0,
			"field": 1}, function() {
				article.published(!article.published());

			});

		return true;
	};

	this.toggleMark = function() {
		var article = this;

		api_request({"op": "updateArticle", "article_ids": this.id(),
			"mode": !article.marked() ? 1 : 0,
			"field": 0}, function() {
				article.marked(!article.marked());

			});

		return true;
	};

	this.expand = function() {
		if (!this.expanded()) {
			this.expanded(true);
		} else {
			this.catchup();
		}
		return true;
	};

	this.read = ko.computed(function() {
		return !this.unread();
	}, this);

	this.pubSrc = ko.computed(function() {
		return this.published() ? "img/pub_set.svg" : "img/pub_unset.svg";
	}, this);

	this.markSrc = ko.computed(function() {
		return this.marked() ? "img/mark_set.svg" : "img/mark_unset.svg";
	}, this);
};

var viewModel = {
	tab: ko.observable(1),
	expanded: ko.observable(true),
	activeFeed: ko.observable(false),
	loadMoreEnabled: ko.observable(true),
	searchQuery: ko.observable(""),
	feeds: ko.observableArray([]),
	headlines: ko.observableArray([]),
	toggleExpanded: function() {
		this.expanded(!this.expanded());
	},
	switchTab: function(id) {
		viewModel.tab(id);
		viewModel.getFeeds();
	},
	sortFeeds: function() {
		if (this.tab() == 1) {
			this.feeds.sort(function(a,b) {
				return a.unread() < b.unread();
			});
		} else if (this.tab() == 2) {
			this.feeds.sort(function(a,b) {
				return a.title().localeCompare(b.title());
			});
		}
	},
	search: function(form) {
		viewModel.searchQuery(form.search.value);
		viewModel.viewFeed(viewModel.activeFeed());
		return false;
	},
	getSelection: function() {
		rv = [];
		for (var i = 0; i < viewModel.headlines().length; i++) {
			if (viewModel.headlines()[i].checked())
				rv.push(viewModel.headlines()[i]);
		}
		return rv;
	},
	getFeeds: function() {
		var unread_only = viewModel.tab() == 1 ? "true" : "false";

		api_request({"op": "getFeeds", "cat_id": -3, "unread_only": unread_only},
			function(data) {
				if (data.status == STATUS_OK) {
					viewModel.feeds.removeAll();

					for (var i = 0; i < data.content.length; i++)
						viewModel.feeds.push(new Feed(data.content[i]));

					viewModel.sortFeeds();
				} else {
					handle_api_error(data);
				}
			});
	},
	catchup: function() {
		var list = viewModel.getSelection();

		if (list.length == 0) list = viewModel.headlines();

		var ids = [];

		if (list.length > 0 && confirm("Mark %d articles as read?".replace("%d", list.length))) {
			for (var i = 0; i < list.length; i++) {
				if (list[i].unread()) {
					list[i].unread(false);
					list[i].checked(false);
					ids.push(list[i].id());
				}
			}

			api_request({"op": "updateArticle", "article_ids": ids.join(","),
				"mode": 0,
				"field": 2}, function() {
					viewModel.getFeeds();
				});
		}

	},
	viewFeed: function(feed) {
		var offset = 0;
		var limit = 15;

		if (feed != viewModel.activeFeed())
			viewModel.searchQuery("");

		if (feed) {
			viewModel.activeFeed(feed);
			viewModel.loadMoreEnabled(true);
		} else {
			feed = viewModel.activeFeed();
			offset = viewModel.headlines().length;
		}

		var search = viewModel.searchQuery();

		api_request({"op":"getHeadlines",  "feed_id": feed.id(),
			"show_content": "true", "show_excerpt": "true", "skip": offset, "search": search,
				"limit": limit, "view_mode": "adaptive"},
			function (data) {
				if (data.status == STATUS_OK) {

					if (offset == 0)
						viewModel.headlines.removeAll();

					for (var i = 0; i < data.content.length; i++) {
						viewModel.headlines.push(new Headline(data.content[i]));
					}

					if (data.content.length < limit)
						viewModel.loadMoreEnabled(false);

					viewModel.getFeeds();

				} else {
					handle_api_error(data);
				}

			});
	},
};

function handle_api_error(data) {
	console.log(data);
	logout();
}

function api_request(params, success) {
	params.sid = sessionStorage['sid'];

	console.log(params);

	$.post(config.api + "/",
			$.toJSON(params),
			success, "json");

}

function logout() {
	sessionStorage['sid'] = false;

	$("#login").show();
	$("#main").hide();

}

function login(username, password) {
	api_request({"op": "login", "user": username, "password": password},
		function(data) {
			if (data.status == STATUS_OK) {

				sessionStorage['sid'] = data.content.session_id;
				sessionStorage['api_level'] = data.content.api_level;

				viewModel.getFeeds();

				$("#login").hide();
				$("#main").show();

			} else {
				handle_api_error(data);
			}

			$("#loginForm")[0].reset();

		});
}

function loginForm() {
	var form = $("#loginForm")[0];

	var username = form.username.value;
	var password = form.password.value;

	login(username, password);

	return false;
}

function timeout() {
	viewModel.getFeeds();
	window.setTimeout("timeout()", 60000);
}

function init() {

	if (sessionStorage['sid'] !== "false") {
		$("#login").hide();
		$("#main").show();
		viewModel.getFeeds();
	}

	if (typeof config == "undefined") {
		$("#fatal").html("Configuration not found, copy config.js-dist to config.js").show();
		$("#login").hide();
		$("#main").hide();
	} else {
		ko.applyBindings(viewModel);

		viewModel.expanded(config.expanded);
	}

	window.setTimeout("timeout()", 60000);
}
